// $Id: $
/**
* @author:        Avi Mehta
* @description:   A library of general functions used in the Form Builder
*                 Module.
* @license:       GPL
*/




/**
* @name:        addComponents()
* @description: Adds the buttons to the DOM based on js array: properties.
*               It was populated on server while this page was being created.
* @params:      none
*/
function addComponents() {
  var j = 0;
  for (var i in properties) {
    $('#leftCol').append('<div class="formbuilderButton" onclick="addItem(\''+i+'\',\''+i+'\'+itemCount,false)">' + i + '</div>');
  }  
}

/**
* @name:        getElementCode(itemId)
* @description: return the basic code for an element
* @params:      itemId: id of the element to be created
*/
function getElementCode(itemId, itemType, itemName) {
  return '<div class="formRow" id="'+itemId+'"><span style="float:left;" class="formbuilderElementDesc">'+itemName+' : '+itemId+' ('+itemType+')</span><img class="close" id="close'+itemId+'" src="'+basePath+'/images/formElements/cross2.gif" alt="Close" /><img class="handle" src="'+basePath+'/images/formElements/move.gif" alt="move" />&nbsp;</div>';
}

/**
* @name:        addItem(itemType)
* @description: When a particular button is pressed, this function adds 
*               the corresponding widgets to the preview column.
* @params:      itemType (refers to the type from the component array)
*/
function addItem(i, itemId, existingItem) {
  if (!existingItem) {
    form[itemId] = new clone(properties[i]);
  }
  var code = getElementCode(itemId, i, form[itemId]['#title'])

  if (itemCount == 0) {
    //First Item
    $("#editingTab").html(code);
  }
  else { //All subsequent Items.
    $("#editingTab").append(code);
  }


  $("#close"+itemId).click(function(){ removeItem( (this.id).substring(5));});
  $("#"+itemId).click(function(){ $(".formRow").removeClass("outline"); $(this).addClass("outline"); showProperties(this.id);});


  createSortables();
  createDropable(itemId);

  //$('#editingTab').SortableAddItem(document.getElementById(itemId)); //Error!!!
  itemCount++;
  recolor();
  $(".formRow").removeClass("outline");
  $('#'+itemId).addClass("outline")
  showProperties(itemId)
  return itemId;

}


/**************************************************************************
* @name:        recolor()
* @description: Recolors the form elements in alternating colors.
* @params:      none.
****************************************************************************/
function recolor() {
  $('.formRow').removeClass('rowAlt').removeClass('row').filter(':odd').addClass('row').end().filter(':even').addClass('rowAlt').end();
  $('.close').filter(':odd').attr('src', basePath+'/images/formElements/cross1.jpg').end().filter(':even').attr('src', basePath+'/images/formElements/cross2.gif').end();
  $('.handle').filter(':odd').attr('src', basePath+'/images/formElements/move.jpg').end().filter(':even').attr('src', basePath+'/images/formElements/move.gif').end();
  putWeights();
}


/**************************************************************************
* @name:        removeItem(item)
* @description: Removes the item from the form array. The corresponding
*               part from the preview column has already been deleted 
*               before this function was called. Adjust third column
*               accordingly
* @params:      item: The item to be removed.
****************************************************************************/
function removeItem(formItem) {
  if (jQuery.className.has($('#' + formItem)[0],'outline')) {
    showFormProperties();
  }
  $('#' + formItem).remove();
  createSortables();
  recolor();
  //Remove the details from the JS $form array
  delete form[formItem];
}


/**************************************************************************
* @name:        updateFormProperty(newId)
* @description: Update the form properties
* @params:      value: the new form property.
*               prop: the name of the property
****************************************************************************/
function updateFormProperty(value, prop) {
  switch (prop) {

    case "id": 
      form['#id'] = value;
      break;
    case "prefix": 
      form['#prefix'] = value;
      break;
    case "suffix": 
      form['#suffix'] = value;
      break;
    case "method": 
      form['#method'] = value;
      break;
    case "action": 
      form['#action'] = value;
      break;
    case "value": 
      form['#value'] = value;
      break;
  }
  
}

/**************************************************************************
* @name:        showFormProperties(newId)
* @description: Show a form to edit the form properties in 3rd column.
* @params:      none
****************************************************************************/
function showFormProperties() {
  var code = '<p> Form Properties: </p><table><tr><td>Form Id: </td><td><input type="text" name="formId" id="formId" size="10" value="'+form['#id']+'" onchange="updateFormProperty(this.value,\'id\');" /></td></tr><tr><td>Prefix: </td><td><input type="text" name="formPrefix" id="formPrefix" size="10" value="'+form['#prefix']+'" onchange="updateFormProperty(this.value,\'prefix\');" /></td></tr><tr><td>Suffix: </td><td><input type="text" name="formSuffix" id="formSuffix" size="10" value="'+form['#suffix']+'" onchange="updateFormProperty(this.value,\'suffix\');" /></td></tr><tr><td>Method: </td><td><input type="text" name="formMethod" id="formMethod" size="10" value="'+form['#method']+'" onchange="updateFormProperty(this.value,\'method\');" /></td></tr><tr><td>Value: </td><td><input type="text" name="formValue" id="formValue" size="10" value="'+form['#value']+'" onchange="updateFormProperty(this.value,\'value\');" /></td></tr><tr><td>Action: </td><td><input type="text" name="formAction" id="formAction" size="10" value="'+form['#action']+'" onchange="updateFormProperty(this.value,\'action\');" /></td></tr></table> <p><br />Add some elements to the form and then click on an element to modify the properties. </p>';
  $("#formbuilder_properties").html(code);
  $(".formRow").removeClass("outline"); 
}


/**************************************************************************
* @name:        showProperties(item)
* @description: Creates the third column for the selected form item. It 
*               uses the keys in the Form array to show the properties. 
* @params:      item: The item whose properties are to be displayed.
****************************************************************************/
function showProperties(formItem) {
  // Everything is displayed in a table.
  var code = '<form name="formProperties" id="formProperties"><table>';

  //Adding option to change element name
  code += '<div class="propertyRow"><tr><td  class="propertyCell">Element Type: </td><td>'+ form[formItem]['#type']+'<td></tr></div><div class="propertyRow"><tr><td  class="propertyCell"><label>Name: </td><td><input type="text" name="formbuilderName" size="10" value="'+formItem+'" onchange="synchronize(\''+formItem+'\',\'#formbuilderName\', 3)" /></div></label></td></tr></div>'
  for (key in form[formItem]) {
    //Insert code for the particular property of widget.
    code += '<div class="propertyRow">';
    switch (key) {
      case '#maxlength':
        code += '<tr><td>Max. Length: </td><td><input type="text" name="maxlength" size="5" value="'+form[formItem][key]+'" onchange="synchronize(\''+formItem+'\', \''+ key+'\' , 3)" /></td></tr>';
      break;
      case '#title':
        code += '<tr><td  class="propertyCell">Title: </td><td><input type="text" name="title" size="10" value="'+form[formItem][key]+'" onchange="synchronize(\''+formItem+'\',\''+ key+'\', 3)" /></td></tr>';
      break;
      case '#size':
        code += '<tr><td  class="propertyCell">Size: </td><td><input type="text" name="size" size="5"  value="'+form[formItem][key]+'" onchange="synchronize(\''+formItem+'\',\''+ key+'\', 3)" /></td></tr>';
      break;
      case '#autocomplete_path':
        code += '<tr><td  class="propertyCell">Autocomplete: </td><td><input type="checkbox" name="autocomplete_path" size="5" onchange="synchronize(\''+formItem+'\',\''+ key+'\', 3)" /></td></tr>';
      break;
      case '#default_value':
        code += '<tr><td  class="propertyCell">Default: </td><td><input type="text" name="default_value"  value="'+form[formItem][key]+'" size="10" onchange="synchronize(\''+formItem+'\',\''+ key+'\', 3)" /></td></tr>';
      break;
      case '#description':
        code += '<tr><td  class="propertyCell">Description: </td><td><textarea name="description" cols=9 rows=5 onchange="synchronize(\''+formItem+'\',\''+ key+'\', 3)">'+form[formItem][key]+'</textarea></td></tr>';
      break;
      case '#cols':
        code += '<tr><td  class="propertyCell">Columns: </td><td><input type="text" name="cols"  value="'+form[formItem][key]+'" size="5" onchange="synchronize(\''+formItem+'\',\''+ key+'\', 3)" /></td></tr>';
      break;
      case '#rows':
        code += '<tr><td  class="propertyCell">Rows: </td><td><input type="text" name="rows" size="5"  value="'+form[formItem][key]+'" onchange="synchronize(\''+formItem+'\',\''+ key+'\', 3)" /></td></tr>';
      break;
      case '#options':
        //Various parsing related things...
        if (form[formItem]['#type'] == 'radios'  ) {
          code += '<tr><td  class="propertyCell">Options: </td></tr>';
          for (i in  form[formItem][key])
            code += '<tr class="optionProp"><td>&nbsp;</td><td><input type="text" name="option'+i+'" onchange="synchronize(\''+formItem+'\',\''+ key+'\', 3)" value="'+ form[formItem][key][i]+'" size="10" /></td></tr>';
          code += '<tr><td>&nbsp;</td><td><img onclick="less()" src="'+basePath+'/images/formElements/decrProp.jpg"  alt="Less"  />&nbsp;<img onclick="more(\'radios\')" src="'+basePath+'/images/formElements/incrProp.jpg" alt="More" /></td></tr>';
        }
        else if (form[formItem]['#type'] == 'select' || form[formItem]['#type'] == 'checkboxes') {
          code += '<tr><td  class="propertyCell"><label>Options: </label></td></tr>';
          var j=0;
          for (i in  form[formItem][key]) {
            code += '<tr class="optionProp"><td><input type="text" name="key'+j+'" onchange="synchronize(\''+formItem+'\',\''+ key+'\', 3)" value="'+i+'" size="10" /></td><td><input type="text" name="option'+j+'" onchange="synchronize(\''+formItem+'\',\''+ key+'\', 3)" value="'+form[formItem][key][i]+'" size="10" /></td></tr>';
            j++;
          }
          code += '<tr><td>&nbsp;</td><td><img onclick="less()" src="'+basePath+'/images/formElements/decrProp.jpg" alt="Less" /><img onclick="more(\'select\')" src="'+basePath+'/images/formElements/incrProp.jpg"  alt="More" /></td></tr>';
        }
        
      break;
      case '#button_type':
        code += '<tr><td  class="propertyCell">Button type:</td><td><input type="text" name="button_type" size="5"  value="'+form[formItem][key]+'" onchange="synchronize(\''+formItem+'\',\''+ key+'\', 3)" /></td></tr>';
      break;
      case '#value':
        code += '<tr><td  class="propertyCell">Value: </td><td><input type="text" name="value" size="5"  value="'+form[formItem][key]+'" onchange="synchronize(\''+formItem+'\',\''+ key+'\', 3)" /></td></tr>';
      break;
      case '#submit':
        code += '<tr><td  class="propertyCell">Submit: </td><td><input type="checkbox" name="submit" onchange="synchronize(\''+formItem+'\',\''+ key+'\', 3)" /></td></tr>';
      break;
      case '#prefix':
        code += '<tr><td  class="propertyCell">Prefix: </td><td><input type="textfield" name="prefix" size="10" onchange="synchronize(\''+formItem+'\',\''+ key+'\', 3)" /></td></tr>';
      break;
      case '#suffix':
        code += '<tr><td  class="propertyCell">Suffix: </td><td><input type="textfield" name="suffix" size="10" onchange="synchronize(\''+formItem+'\',\''+ key+'\', 3)" /></td></tr>';
      break;
      case '#multiple':
        code += '<tr><td  class="propertyCell">Multiple: </td><td><input type="checkbox" name="multiple" ' + (form[formItem][key]?'checked="checked"':'') + ' onchange="synchronize(\''+formItem+'\',\''+ key+'\', 3)" /></td></tr>';
      break;
      case '#required':
        code += '<tr><td  class="propertyCell">Required: </td><td><input type="checkbox" name="required" ' + (form[formItem][key]?'checked="checked"':'') + ' onchange="synchronize(\''+formItem+'\',\''+ key+'\', 3)" /></td></tr>';
      break;
      case '#attributes':
        code += '<tr><td  class="propertyCell">Attributes: </td><td><img onclick="removeProp(\'attributes\'); synchronize(\''+formItem+'\',\''+ key+'\', 3)" " src="'+basePath+'/images/formElements/decrProp.jpg" /><img onclick="addProp(\'attributes\'); synchronize(\''+formItem+'\',\''+ key+'\', 3)" " src="'+basePath+'/images/formElements/incrProp.jpg" /></td></tr>';
        var val = form[formItem][key];

          for (var i  in val) {
            code += '<tr class="attributes"><td><input type="text"  class="attributes_key" onchange="synchronize(\''+formItem+'\',\''+ key+'\', 3)" value='+i+' size="10" /></td><td><input type="text" class="attributes_option" onchange="synchronize(\''+formItem+'\',\''+ key+'\', 3)" value='+val[i]+' size="10" /></td></tr>';
          }
          code += '<tr class="attributes"><td><input type="text" class="attributes_key" onchange="synchronize(\''+formItem+'\',\''+ key+'\', 3)" value=" " size="10" /></td><td><input type="text"  class="attributes_option" onchange="synchronize(\''+formItem+'\',\''+ key+'\', 3)" value=" " size="10" /></td></tr>'
      break;
      case '#after_build':
        code += '<tr><td  class="propertyCell"><label>After_build: </label></td><td><img onclick="removeProp(\'after_build\'); synchronize(\''+formItem+'\',\''+ key+'\', 3)" src="'+basePath+'/images/formElements/decrProp.jpg" />&nbsp;<img onclick="addProp(\'after_build\'); synchronize(\''+formItem+'\',\''+ key+'\', 3)" src="'+basePath+'/images/formElements/incrProp.jpg" /></td></tr>';
        var val = form[formItem][key];
          for (var i in val) {
            code += '<tr class="after_build"><td>&nbsp;</td><td><input type="text" onchange="synchronize(\''+formItem+'\',\''+ key+'\', 3)" value='+i+' size="10" /></td></tr>';
        }
        code += '<tr class="after_build"><td>&nbsp;</td><td><input type="text" onchange="synchronize(\''+formItem+'\',\''+ key+'\', 3)" value=" " size="10" /></td></tr>';
      break;
      case '#process':
      case '#input' :
         //Internal... needs to be hidden
      break;
      default:
        code += '<tr><td  class="propertyCell">'+key+ '</td><td>Missing</td></tr>';
      break;
    }
    code+= '</div>'
  } 
  code += "</table></form>";
  $("#formbuilder_properties").html(code);
}

function removeProp(property) {
  if ($('.'+property).length == 1) {
    $('.'+property+':last').find('input').val(' ');
  }
  else {
    $('.'+property+':last').remove();
  }
}

function addProp(property) {
  $('.'+property+':last').after('<tr class='+property+'>' + $('.'+property+':last').html() + '</tr>');
}

/**************************************************************************
* @name:      synchronize(formItem, key, source)
* @description:   Function used to synchronize the three "data pools" 
*         namely: DOM Form, JS form Object and properties field.
* @params: 
*   formItem: the form widget that is being synchronized
*   element:  the elementof the widget that is being synchronized
*   source:   1 => js form object, 2 => DOM form, 3 => properties
* 
* Note: Only title uses the source = 2, source = 1 is used when you are 
*     editing an already created form.
****************************************************************************/
function synchronize(formItem, key, source) {
    switch (key)   {
      case '#maxlength':
          form[formItem][key] = $('#formProperties')[0].maxlength.value;
      break;
      case '#title':
            form[formItem][key] = $('#formProperties')[0].title.value;
            $('.outline').find('span:first').html(form[formItem][key] + ' : ' + formItem + ' ('+ form[formItem]['#type'] + ')');
            
      break;
      case '#formbuilderName':
      {
        var newId = $('#formProperties')[0].formbuilderName.value;
        form[newId] = form[formItem];
        $('#'+formItem).after(getElementCode(newId, form[formItem]['#type'], form[formItem]['#title'])).remove();
        delete form[formItem];
        $("#close"+newId).click(function(){ removeItem( (this.id).substring(5));});
        $("#"+newId).click(function(){ $(".formRow").removeClass("outline"); $(this).addClass("outline"); showProperties(this.id);});
        //$('#editingTab').SortableAddItem(document.getElementById(itemId)); //Error!!!
        recolor();
        $(".formRow").removeClass("outline");
        $('#'+newId).addClass("outline")
        showProperties(newId)

      }
      break;
      case '#size':
          form[formItem][key] = $('#formProperties')[0].size.value;
      break;
      case '#autocomplete_path':
          form[formItem][key] = ($('#formProperties')[0].autocomplete_path.checked == 1)? true : false;
      break;
      case '#default_value':
           form[formItem][key] = $('#formProperties')[0].default_value.value;
      break;
      case '#description':
          form[formItem][key] = $('#formProperties')[0].description.value;
      break;
      case '#required':
        form[formItem][key] = $('#formProperties')[0].required.checked
      break;
      case '#multiple':
          if (form[formItem][key] = $('#formProperties')[0].multiple.checked) {
            form[formItem]['#size'] = 2;
            $('#formProperties')[0].size.value = 2;
          }
          else {
            form[formItem]['#size'] = 1;
            $('#formProperties')[0].size.value = 1;
          }
      break;
      case '#cols':
          form[formItem][key] = $('#formProperties')[0].cols.value;
      break;
      case '#rows':
          form[formItem][key] = $('#formProperties')[0].rows.value;
      break;
      case '#options':
        if (source == 3) {
          var value = {};
          if (form[formItem]['#type'] == 'radios') {
            var temp1 = $('input').parents('.optionProp').size();
            for (var i = 0 ; i< temp1 ;i++) {
              value[i] = $('#formProperties')[0]['option'+i].value ;
            } 
          }
          else if (form[formItem]['#type'] == 'checkboxes') {
            var temp1 = $('input').parents('.optionProp').size();            
            for (var i = 0 ; i< temp1 ;i++) {
              value[$('#formProperties')[0]['key'+i].value] =  $('#formProperties')[0]['option'+i].value;
            }
          }
          else if (form[formItem]['#type'] == 'select') {
            var temp1 = $('input').parents('.optionProp').size();
            for (var i = 0 ; i< temp1 ;i++) {
              value[ $('#formProperties')[0]['key'+i].value] = $('#formProperties')[0]['option'+i].value ;
            }
          }
          form[formItem][key] = value;
          
        }
      break;
      case '#prefix':
          form[formItem][key] = $('#formProperties')[0].prefix.value;
      break;
      case '#suffix':
          form[formItem][key] = $('#formProperties')[0].suffix.value;
      break;
      case '#attributes':
          form[formItem][key] = {};
          var temp1 = $('input').parents('.attributes').size();
          var val = [' ',' '];
          var j =0;
          for (var i = 0 ; i< temp1 ;i++) {
            val[0] = $('.attributes_key')[i].value.replace(/^\s*|\s*$/g,"");
            val[1] = $('.attributes_option')[i].value.replace(/^\s*|\s*$/g,"");
            if (val[0] != '' && val[1] != '' && typeof val[1] != "undefined" && typeof val[0] != "undefined"  ) {
              form[formItem][key][val[0]] = val[1];
            }

          }
      break;
      case '#after_build':
          form[formItem][key] = [];
          var temp1 = $('input').parents('.after_build').size();
          var j =0;
          var val;
          for (var i = 0 ; i< temp1 ;i++) {
            val =  $('.after_build').find('input')[i].value.replace(/^\s*|\s*$/g,"")
            if (val != '')
              form[formItem][key][j++] =  val ;
          }
      break;
    }
}


/**************************************************************************
* @name:        createSortables()
* @description: Create the sortables. Called after any element is added.
* @params:      none
****************************************************************************/
function createSortables() {
  $('#editingTab').Sortable(
  {
    accept :        'formRow',
    opacity:        0.5,
    handle:         '.handle',
    onchange:       recolor,
    onStop:       recolor,
    axis :          "vertically",
    helperclass : 	"sorthelper",
    tolerance :     "pointer",
    fx:				      200,
		revert:			    true
  })
}

function createDropable(itemId) {
/*  $('#'+itemId).Droppable(
	{
		accept : 'formRow', 
		activeclass: 'dropactive', 
		hoverclass:	'drophover',
		ondrop:	function (drag) 
				{
					alert("into: " + this.id); //the droppable
					alert("droping: " + drag.id); //the dragganle
				},
          
    tolerance : "intersect"
	}
);*/

}


/**************************************************************************
* @name:        less()
* @description: Removes the last option/radiobutton/checkbox from the 
*               currently selected formtem.
* @params:      none
****************************************************************************/
function less() {
  $('.optionProp:last',$('#formProperties')[0]).remove();
  $('.outline').find('label:last').remove(); // How do i remove the warning?? There are labels in case of checkboxes and radio buttons but there are none in case of list.
  synchronize($('.outline').get(0).id,'#options',3);
}

/**************************************************************************
* @name:        more(widget)
* @description: Adds more options/radiobuttons/checkboxes to the
*               currently selected widget
* @params:      widget: the widget to which option/radiobutton/checkbox
*               is added.
****************************************************************************/
function more(widget) {
  if (widget == "after_build") {
    
  }
  else if (widget == 'radios' || widget == 'checkboxes') {
    var prev = $('input').parents('.optionProp').size()-1;
    $('.optionProp:last',$('#formProperties')[0]).after('<tr class="optionProp"><td>&nbsp;</td><td><input type="text" name="'+'option'+(prev+1)+'" value="'+$('#formProperties')[0]['option'+prev].value+'" size="10" /></td></tr>');
    $('#formProperties')[0]['option'+(prev+1)].onchange = $('#formProperties')[0]['option'+prev].onchange;
    $('.outline').find('br:last').after('<label>'+$('.outline').find('label:last').html()+'</label><br />');
    
    synchronize($('.outline').get(0).id,'#options',3);
  }
  else {   
    var prev = $('input').parents('.optionProp').size()-1;
    $('.optionProp:last',$('#formProperties')[0]).after('<tr class="optionProp"><td><input type="text" name="'+'key'+(prev+1)+'" value="'+$('#formProperties')[0]['key'+prev].value+'" size="10" /></td><td><input type="text" name="'+'option'+(prev+1)+'" value="'+$('#formProperties')[0]['option'+prev].value+'" size="10" /></td></tr>');
    $('#formProperties')[0]['option'+(prev+1)].onchange = $('#formProperties')[0]['option'+prev].onchange;
    $('#formProperties')[0]['key'+(prev+1)].onchange = $('#formProperties')[0]['key'+prev].onchange;
    
    synchronize($('.outline').get(0).id,'#options',3);
  }
}


/**************************************************************************
* @name:        setClickable(id)
* @description: Makes the id passed in parameter to be editable in-line.
* @params:      id: The id of element to be made editable
****************************************************************************/
function setClickable(id) {
  $('#'+id).click(function() {
    if ($(this).html() == "Edit") {
      $(this).html('');
    }
    var textarea = '<div id="editorCont'+id+'" ><input class="labelClass" id="editor'+id+'" type="text" size="15" name="textfield" value="'+$(this).html()+'" /></div>';
    $(this).after(textarea).remove(); // Add the textfield and remove label
    $('#editor'+id).blur(function(){saveChanges(this, id);}); //Save changes upon blur
    $('#editor'+id)[0].focus();     // Bring to focus
  })
  .mouseover(function() {
    $(this).addClass("editable");   //Highlight
  })
  .mouseout(function() {
    $(this).removeClass("editable");  //Unhighlight :)
  });
}


/**************************************************************************
* @name:        saveChanges(object, id)
* @description: Called when the focus is removed from the textfield. It
*               saves the changes to the label widget and removes 
*               textfield.
* @params:      object: not currently used.
*               id: Id of the editabe field to be saved.
****************************************************************************/
function saveChanges(obj, id) {
  var t = $("#editor"+id).val();

  if (t == '' ) {
    t = "Edit";
  }
  $("#editorCont"+id).after('<div class="labelClass" id="'+id+'">'+t+'</div>').remove() ;
  $('#'+id).click(function(){ $(".formRow").removeClass("outline");  $('#'+this.id.substring(6)).addClass("outline"); showProperties(this.id.substring(6));});
  setClickable(id);
  synchronize(id.substring(6),'#title',2);
}

/**************************************************************************
* @name:        clone(original)
* @description: Clones the paramenter and adds to the calling object.
* @params:      original: the object to be copied.
****************************************************************************/
function clone(original) {
    for (i in original) {
        if (typeof original[i] == 'object') {
            this[i] = new clone(original[i]);
        }
        else
            this[i] = original[i];
    }
}



    var m = {
            '\b': '\\b',
            '\t': '\\t',
            '\n': '\\n',
            '\f': '\\f',
            '\r': '\\r',
            '"' : '\\"',
            '\\': '\\\\'
        },
        s = {
            array: function (x) {
                var a = ['['], b, f, i, l = x.length, v;
                for (i = 0; i < l; i += 1) {
                    v = x[i];
                    f = s[typeof v];
                    if (f) {
                        v = f(v);
                        if (typeof v == 'string') {
                            if (b) {
                                a[a.length] = ',';
                            }
                            a[a.length] = v;
                            b = true;
                        }
                    }
                }
                a[a.length] = ']';
                return a.join('');
            },
            'boolean': function (x) {
                return String(x);
            },
            'null': function (x) {
                return "null";
            },
            number: function (x) {
                return isFinite(x) ? String(x) : 'null';
            },
            object: function (x) {
                if (x) {
                    if (x instanceof Array) {
                        return s.array(x);
                    }
                    var a = ['{'], b, f, i, v;
                    for (i in x) {
                        v = x[i];
                        f = s[typeof v];
                        if (f) {
                            v = f(v);
                            if (typeof v == 'string') {
                                if (b) {
                                    a[a.length] = ',';
                                }
                                a.push(s.string(i), ':', v);
                                b = true;
                            }
                        }
                    }
                    a[a.length] = '}';
                    return a.join('');
                }
                return 'null';
            },
            string: function (x) {
                if (/["\\\x00-\x1f]/.test(x)) {
                    x = x.replace(/([\x00-\x1f\\"])/g, function(a, b) {
                        var c = m[b];
                        if (c) {
                            return c;
                        }
                        c = b.charCodeAt();
                        return '\\u00' +
                            Math.floor(c / 16).toString(16) +
                            (c % 16).toString(16);
                    });
                }
                return '"' + x + '"';
            }
        };

String.prototype.parseJSON = function () {
    try {
        return !(/[^,:{}\[\]0-9.\-+Eaeflnr-u \n\r\t]/.test(
                this.replace(/"(\\.|[^"\\])*"/g, ''))) &&
            eval('(' + this + ')');
    } catch (e) {
        return false;
    }
};

/**************************************************************************
* @name:        createForms()
* @description: Create the main UI for creating forms
* @params:      none
****************************************************************************/
function createForms() {
  $('#displayPane').hide();
  $('#mainMenu1').hide();
  if (itemCount == 0 ||  ( itemCount != 0 && confirm('There is already a form in the memory. Press OK to erase it and Cancel to continue from previous Form?'))) {
    //Form Menu
    $('#formbuilder_wrapper').html('<div id="mainMenu2"><a href="#" class="formbuilder_link" onclick="$(\'#formbuilder_wrapper\').fadeOut();$(\'#mainMenu1\').show();">Back</a><span>|</span><a href="#" class="formbuilder_link"  onclick="send()">Save</a><span>|</span><a href="#" class="formbuilder_link" onclick="createForms()">Reset</a>  <span>|</span><a href="#" class="formbuilder_link" onclick="showFormProperties()">Form Properties</a></div><table> <tr><td class="formbuilder_columnHead">Components</td><td><div class="formbuilder_centered"> <span class="formbuilderTab" onclick="showTab(\'editing\')">Editing</span><span class="formbuilderTab" onclick="showTab(\'phpCode\')">PHP code</span><span class="formbuilderTab" onclick="showTab(\'htmlPrev\')">HTML Preview</span></div></td><td class="formbuilder_columnHead"> Properties</td></tr><tr><td></td><td></td><td></td></tr><tr><td id="buttons" class="columnSide"><div id="leftCol"></div> </td><td id="preview">  <div id="editingTab">Select components of the form from the Left Column  </div><div id="phpCodeTab"></div><div id="htmlTab"></div> </td><td id="formbuilder_properties" class="columnSide"><div><p> Form Properties: </p><table><tr><td>Form Id: </td><td><input type="text" name="formId" id="formId" size="10" value="'+form['#id']+'" onchange="updateFormProperty(this.value,\'id\');" /></td></tr><tr><td>Prefix: </td><td><input type="text" name="formPrefix" id="formPrefix" size="10" value="'+form['#prefix']+'" onchange="updateFormProperty(this.value,\'prefix\');" /></td></tr><tr><td>Suffix: </td><td><input type="text" name="formSuffix" id="formSuffix" size="10" value="'+form['#suffix']+'" onchange="updateFormProperty(this.value,\'suffix\');" /></td></tr><tr><td>Method: </td><td><input type="text" name="formMethod" id="formMethod" size="10" value="'+form['#method']+'" onchange="updateFormProperty(this.value,\'method\');" /></td></tr><tr><td>Value: </td><td><input type="text" name="formValue" id="formValue" size="10" value="'+form['#value']+'" onchange="updateFormProperty(this.value,\'value\');" /></td></tr><tr><td>Action: </td><td><input type="text" name="formAction" id="formAction" size="10" value="'+form['#action']+'" onchange="updateFormProperty(this.value,\'action\');" /></td></tr></table> <p><br />Add some elements to the form and then click on an element to modify the properties. </p></div></td></tr><tr><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td> </tr></table>').fadeIn();
    initializeForm();
    addComponents();
  }
  else {
    $('#formbuilder_wrapper').fadeIn();
  }
}

/**************************************************************************
* @name:        showTab()
* @description: Show a particular tab.
* @params:      type: tab to be shown.
****************************************************************************/
function showTab(type) {
  if (type == 'editing') {
    $('#editingTab').show();
    $('#htmlTab').hide();
    $('#phpCodeTab').hide();
    return;
  }

  
  $('#editingTab').hide();
  $('#htmlTab').hide();
  $('#phpCodeTab').hide();

  if (type == 'htmlPrev')
  {
    $('#htmlTab').show();
  }
  else {
    $('#phpCodeTab').show();
  }
  $('#loadingAnimation').show();
  $.post("?q=formbuilder/views",{
    "action" : type,
    "form"   : s.object(form),
    "rand" : Math.random()
    },function(xml){  $('#loadingAnimation').hide();
      if ($('#htmlTab').is(':visible')) {
        $('#htmlTab').html(xml);
      }
      else {
        $('#phpCodeTab').html(xml);
      }

    });
  

}

/**
* @name:        editRequest()
* @description: Post a request to server to edit a already created form
* @params:      formid: the id of the form to be edited.
*/
function editRequest(formid) {
  $('#loadingAnimation').show();
  $.post("?q=formbuilder/action",{
    "action" : 'edit',
    "formId" : formid,
    "rand" : Math.random()
    },function(xml){  $('#loadingAnimation').hide(); createForm(xml);});
}
  

/**
* @name:        manageForms()
* @description: Show a screen to manage all the created forms!!!
* @params:      data: the details of the form ids coming from server via AJAX
*/
function manageForms(data) {
  $('#loadingAnimation').hide();
  if (data.substring(0,5) == "Error") {
    alert("Some error occurred!\n\n Error Description: " + data.substring(5));
    return;
  }
  var a = eval(data);
  var disp = '<table width="100%">';
  var l = a.length;
  for (var i =0; i <l;i++) {
    disp += '<tr id="row_'+a[i]+'"><td width="20%" >'+a[i]+'</td><td width="20%" style="cursor:pointer; border-right:1px solid; border-left:1px solid;"><a href="#" onclick="renameRequest(\''+a[i]+'\'); return false;">Rename</a></td><td width="20%" style="cursor:pointer; border-right:1px solid;"><a href="#" onclick="editRequest(\''+a[i]+'\'); return false;">Edit</a></td><td width="20%" style="cursor:pointer;"><a href="#" onclick="deleteRequest(\''+a[i]+'\'); return false;">Delete</a></td></tr>';
  }
  disp += "</table>";
  $('#displayPane').html(disp).fadeIn();
  $('#displayPane').find('tr:even').attr('class', 'rowAlt');
  $('#displayPane').find('tr:odd').attr('class', 'row');

}
 

/**************************************************************************
* @name:        manageRequest()
* @description: post a request via AJAX to the server to download the list 
*               of forms available. 
* @params:      none
****************************************************************************/
function manageRequest() {
  if ($('#displayPane').is(':visible')) {
    $('#displayPane').fadeOut();
  }
  else {
    $('#loadingAnimation').show();
    $.post("?q=formbuilder/list",{
      "rand" : Math.random()
      },manageForms);
  }
}

/**************************************************************************
* @name:        deleteRequest()
* @description: post a request via AJAX to the server to delete a form.
* @params:      formid: the id of the form to be deleted.
****************************************************************************/
function deleteRequest(formid) {
  $('#loadingAnimation').show();
  $.post("?q=formbuilder/action",{
    "formId" : formid,
    "action" : "delete",
    "rand" : Math.random()
    },function(xml){  $('#loadingAnimation').hide(); if (xml != "Error") $('#row_'+xml).remove();  $('#displayPane').find('tr:even').attr('class', 'rowAlt');  $('#displayPane').find('tr:odd').attr('class', 'row');});
}

/**************************************************************************
* @name:        renameRequest()
* @description: post a request via AJAX to the server to rename a form.
* @params:      formid: the id of the form to be renamed.
****************************************************************************/
function renameRequest(formid) {
  
  newId = window.prompt("Enter a new form Id");
  if (!newId) return;
  $('#loadingAnimation').show();
  $.post("?q=formbuilder/action",{
    "formId" : formid,
    "newFormId" : newId,
    "action" : "rename",
    "rand" : Math.random()
    },function(xml){  $('#loadingAnimation').hide(); if (xml.substring(0,1) == "["){ var a = eval(xml);  $('#row_'+a[1]).after('<tr id="row_'+a[0]+'"><td width="30%" >'+a[0]+'</td><td width="20%" style="cursor:pointer; border-right:1px solid; border-left:1px solid;"><a href="#"  onclick="renameRequest(\''+a[0]+'\'); return false;">Rename</a></td><td width="20%" style="cursor:pointer; border-right:1px solid;"><a href="#" onclick="editRequest(\''+a[0]+'\'); return false;">Edit</a></td><td width="20%" style="cursor:pointer;"><a href="#" onclick="deleteRequest(\''+a[0]+'\'); return false;">Delete</a></td></tr>').remove(); $('#displayPane').find('tr:even').attr('class', 'rowAlt');$('#displayPane').find('tr:odd').attr('class', 'row');}  else alert("Some error occurred!\n\n Error Description: " + xml.substring(5)); });
}

/**************************************************************************
* @name:        initializeForm()
* @description: Initialize the form variables!
* @params:      none
****************************************************************************/
function initializeForm() {
  form = {
              "#id" : "new_form",
              "#value"  : "",
              "#prefix" : "",
              "#suffix" : "",
              "#method" : "",
              "#action" : ""
              };
  itemCount = 0;
}


/**************************************************************************
* @name:        createForm()
* @description: Initialize the form variables from the data received via AJAX
* @params:      data: form details 
****************************************************************************/
function createForm(data) {
  createForms();
  eval("var tempForm = " + data);
  var itemId
  for (var item in tempForm)   {
    if (item.substring(0,1) == "#") {
      form[item] = tempForm[item];    // continue in case of form properties
      continue;
    }

    form[item] = tempForm[item];
    itemId = addItem(tempForm[item]['#type'],item,true);
  }
 
  showFormProperties();

}

/**
 * @name:        openForm()
 * @description: Gets a form from a URL and opens the form in editor.
 * @param        url: Link where the form will be found.
 *               formId : the id of the form.
 */
function openForm(url, formId) {

 $.get("?q="+url+"&formbuilder_form="+formId,createForm);
}

/*undocumented function :)*/
function test()
{
 $.post("?q=formbuilder/views",{
    "action" : "getDefault",
    "elementType" : 'submit',
    "rand" : Math.random()
    },function(xml){  alert(xml); });
}


/**************************************************************************
* @name:        putWeights()
* @description: Put weights in form elements to reflect their order.
*               This has to be changed and done without using '#weights' 
*                property.
* @params:      data: form details 
****************************************************************************/
function putWeights()
{
  var serial = $.SortSerialize('editingTab');
  var newForm = {};
  for (var item in form)   {
    if (item.substring(0,1) == "#") {
      newForm[item] = form[item];
      continue;
    }
  }

  serial = serial.hash.split("&")
  //  alert(serial)
  for(var i =0; i < serial.length;i++) {
    newForm[serial[i].substring(serial[i].indexOf('=')+1)] = form[serial[i].substring(serial[i].indexOf('=')+1)];
  }

  form = newForm;

}
